/***( dllist.h )****************************************************************
*                                                                              *
*  Contains header info for generic doubly-linked list functions.              *
*                                                                              *
********************************************************************************
*                                                                              *
*  Written: Dec  7, 1989 - SBF                                                 *
*  Updated: MMM DD, YYYY - XXX (                                            )  *
*                                                                              *
*******************************************************************************/

#define MAXNUMDLL   64                        /* Maximum number of open lists */

#define SEL_SINGLE   0              /* Single selection mode for dll_scroll() */
#define SEL_MULTI   1             /* Multiple selection mode for dll_scroll() */

#define ADD_SORT   0      /* Add entry in sorted order using compare function */
#define ADD_NOSORT   1    /* Add entry with no sorting using compare function */

# if defined (RT) || defined (NCR)
typedef char * generic;                              /* Generic pointer type */
# else
typedef void generic;                                /* Generic pointer type */
# endif

struct dll_entry                                    /* Generic list structure */
{
	struct dll_entry *dll_next;                  /* Pointer to next list entry */
	struct dll_entry *dll_prev;              /* Pointer to previous list entry */
	generic *dll_ptr;                                       /* Pointer to data */
};

struct dll_cb      /* Control block structure to keep track of multiple lists */
{
	struct dll_entry *dll_head;                     /* Pointer to head of list */
	struct dll_entry *dll_curr;               /* Pointer to current list entry */
# ifdef ANSI
	int (*dll_cmp)(generic *, generic *);    /* Pointer to comparison function */
# else
	int (*dll_cmp)();
# endif
	int dll_size;                                         /* Size of list data */
};

struct sel_entry
{
	generic *sel_data;
};

#ifdef ANSI
int dll_add(int, generic *, int);
int dll_close(int);
generic *dll_curr(int);
int dll_del(int);
generic *dll_find(int, generic *);
struct dll_entry *dll_getpos(int);
generic *dll_next(int);
int dll_open(int (*)(), int);
generic *dll_prev(int);
int dll_rebuild(int, int (*)());
generic *dll_seek(int, int, int);
int dll_setpos(int, struct dll_entry *);

int dll_scroll(int, int, int, int, int *, int, char *(*)(), int, int, int, int, int, ...);
int dll_scroll_close(int *);
#else
int dll_add();
int dll_close();
generic *dll_curr();
int dll_del();
generic *dll_find();
struct dll_entry *dll_getpos();
generic *dll_next();
int dll_open();
generic *dll_prev();
int dll_rebuild();
generic *dll_seek();
int dll_setpos();

int dll_scroll();
int dll_scroll_close();
#endif
