#define NAMELEN 14	/* Max length of user name and password */
#define PWDLEN 11
#define MODLEN 14		/* Max length of module name - corresponds to max 		*/
							/* length of a filename in operating system				*/
#define PERMLEN 4		/* Max length of permissions string 						*/
							/*  - presently allows for 'a', 'c', 'd', 'i'			*/

# define PASSKEY  "password"

/*
 * The following are used for space allocation - they must be at least the
 * size of the corresponding parameter above plus one.
*/
#define NAMEARLEN 16
#define PWDARLEN 16
#define MODARLEN 16
#define PERMARLEN 5

typedef struct user_entry
{
	char user[NAMEARLEN];
	char pwd[PWDARLEN];
} USER_ENT;

typedef struct mod_entry
{
	char mod[MODARLEN];
	char perm[PERMARLEN];
	char user[NAMEARLEN];
	int writeflag;
} MOD_ENT;


/*******************************
*  Password Permissions flags  *
*******************************/
#define PERM_INIT         0
#define PERM_NOTMPFILE    1
#define PERM_LOCKED       2
#define PERM_NOFILE       4
#define PERM_NOACC        8
#define PERM_INVFILE     16
#define PERM_NONE        32
#define PERM_EXISTS      64
#define PERM_ADD        128
#define PERM_CHANGE     256
#define PERM_DELETE     512
#define PERM_INQUIRE   1024
#define PERM_DEFAULT   2047

typedef struct perms
{
	char         code;
	unsigned int perms;
} PERMS;


#ifdef UNIX
#define OPN_FILE(fname, mode)    open(fname, mode);
#define PCRT_FILE(fname)         open(fname, O_CREAT | O_RDWR, 00666);
#define TCRT_FILE(fname)         open(fname, O_CREAT | O_EXCL | O_RDWR, 00644);
#endif

#ifdef MSC
#define  OPN_FILE(fname, mode)    open(fname, O_TEXT | O_RDWR )
#define  PCRT_FILE(fname)         open(fname, O_CREAT | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#define  TCRT_FILE(fname)         open(fname, O_CREAT | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#endif

#ifdef __TURBOC__
#define  OPN_FILE(fname, mode)    open(fname, O_BINARY | O_RDWR)
#define  PCRT_FILE(fname)         open(fname, O_CREAT  | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#define  TCRT_FILE(fname)         open(fname, O_CREAT | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#endif

#ifdef LATTICE
#define  OPN_FILE(fname, mode)    open(fname, O_RAW | O_RDWR)
#define  CRT_FILE(fname)          open(fname, O_CREAT | O_RAW | O_RDWR)
#endif

#ifdef __WATCOMC__
#define  OPN_FILE(fname, mode)    open(fname, O_TEXT | O_RDWR )
#define  PCRT_FILE(fname)         open(fname, O_CREAT | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#define  TCRT_FILE(fname)         open(fname, O_CREAT | O_TEXT | O_RDWR, S_IREAD | S_IWRITE)
#endif

#ifdef __ZTC__
#define  OPN_FILE(fname, mode)    open(fname, O_RDWR)
#define  CRT_FILE(fname)          creat(fname, O_RDWR)
#endif




#ifdef ANSI
/* MODFILE.C */
void bin_to_text_64(int [], unsigned char *);
void text_to_bin_64(unsigned char *, int []);
void mod_file(char *);
void initpwdscreen(char *);
void setvars(char *);
int openpwdfile(void);
void pwdscreenmaint(void);
void closepwdfile(void);
void display_extras_passwd(void);
void pswd_add_mode(void);
void pswd_change_mode(void);
void pswd_delete_mode(void);
void pswd_inquire_mode(void);
void add_user(void);
void add_module(void);
void chg_user(void);
void chg_module(void);
void del_user(void);
void del_module(void);
void inq_user(void);
void inq_module(void);
void getperms(int, int, char *, int);
void get_pwd( int, int, int, char * );
void clear_window(void);
void get_module_from_list( int, int, int, int, int );
void get_user_from_list( int, int, int, int, int );
int cmpperms(char *, char *);
char *user_dsp(struct user_entry *);
char *mod_dsp(struct mod_entry *);
char *userperm_dsp(struct mod_entry *);
char *modperm_dsp(struct mod_entry *);
/* MFOPS.C */
int readfile(void);
int validfile(FILE *);
int writefile(void);
void writelist(FILE *);
void writeusers(FILE *);
void writemods(FILE *);
void writehdr(FILE *);
int usercmp(struct user_entry *, struct user_entry *);
int modcmp(struct mod_entry *, struct mod_entry *);
/* LOGINMOD.C */
unsigned int checkperms(char *, int, char *[], char *, char *);
void input_pwd(char *, char *);
char *basename(char *);
int retry_win(void);
void chg_win(char *, char *);
/* LFOPS.C */
unsigned int passwd(char *, char *, char *, char *);
char *parsename(FILE *);
char *parsestr(FILE *);
int eatspace(FILE *);
int eatline(FILE *);
int parseuser(FILE *, char *, char *);
unsigned int parseperms(FILE *, char *, char *);
void lower(register char *);
void upper(register char *);
int cistrcmp(char *, char *);
/* CRYPT.C */
void setkey(char *);
/* CRC.C */
long crc_calc(FILE *);
#else
/* MODFILE.C */
void bin_to_text_64();
void text_to_bin_64();
void mod_file();
void initpwdscreen();
void setvars();
int openpwdfile();
void pwdscreenmaint();
void closepwdfile();
void display_extras_passwd();
void pswd_add_mode();
void pswd_change_mode();
void pswd_delete_mode();
void pswd_inquire_mode();
void add_user();
void add_module();
void chg_user();
void chg_module();
void del_user();
void del_module();
void inq_user();
void inq_module();
void getperms();
void get_pwd();
void clear_window();
void get_module_from_list();
void get_user_from_list();
int cmpperms();
char *user_dsp();
char *mod_dsp();
char *userperm_dsp();
char *modperm_dsp();
/* MFOPS.C */
int readfile();
int validfile();
void deluserfmod();
int writefile();
void writelist();
void writeusers();
void writemods();
void writehdr();
int usercmp();
int modcmp();
/* LOGINMOD.C */
unsigned int checkperms();
void input_pwd();
char *basename();
int retry_win();
void chg_win();
/* LFOPS.C */
unsigned int passwd();
char *parsename();
char *parsestr();
int eatspace();
int eatline();
int parseuser();
unsigned int parseperms();
void lower();
void upper();
int cistrcmp();
/* CRYPT.C */
void setkey();
/* CRC.C */
long crc_calc();
#endif

